/* vim: set sw=4 sts=4 et foldmethod=syntax : */

/*
 * Copyright (c) 2006 Ciaran McCreesh <ciaranm@ciaranm.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright notice,
 *       this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the library nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <libwrapiter/libwrapiter.hh>
#include <test/test_framework.hh>
#include <test/test_runner.hh>
#include <set>
#include <vector>
#include <map>

using namespace test;
using namespace libwrapiter;

/**
 * \file
 * Test cases for libwrapiter_forward_iterator.hh .
 */

namespace test_cases
{
    /**
     * Test class.
     */
    class MyClass
    {
        private:
            MyClass(const MyClass &);
            const MyClass & operator= (const MyClass &);

            struct Impl;
            Impl * _impl;

        public:
            MyClass();
            ~MyClass();

            typedef libwrapiter::ForwardIterator<MyClass, int> Iterator;
            Iterator begin() const;
            Iterator end() const;
    };

    /**
     * Test pair class.
     */
    class MyPairClass
    {
        private:
            MyPairClass(const MyPairClass &);
            const MyPairClass & operator= (const MyPairClass &);

            struct Impl;
            Impl * _impl;

        public:
            MyPairClass();
            ~MyPairClass();

            typedef libwrapiter::ForwardIterator<MyClass, std::pair<const std::string, std::string> > Iterator;
            Iterator begin() const;
            Iterator end() const;
    };

    /**
     * \test libwrapiter tests.
     */
    struct LibWrapIterEmptyTest : TestCase
    {
        LibWrapIterEmptyTest() : TestCase("libwrapiter empty test") { }

        void run()
        {
            MyClass c;
            MyClass::Iterator i(c.begin()), i_end(c.end());
            TEST_CHECK(i == i_end);
            TEST_CHECK(! (i != i_end));
            TEST_CHECK(i == i);
            TEST_CHECK(! (i != i));
            TEST_CHECK(i_end == i_end);
            TEST_CHECK(! (i_end != i_end));
        }
    } test_case_libwrapiter_empty;

    /**
     * \test libwrapiter tests.
     */
    struct LibWrapIterEmptyPairTest : TestCase
    {
        LibWrapIterEmptyPairTest() : TestCase("libwrapiter empty pair test") { }

        void run()
        {
            MyPairClass c;
            MyPairClass::Iterator i(c.begin()), i_end(c.end());
            TEST_CHECK(i == i_end);
            TEST_CHECK(! (i != i_end));
            TEST_CHECK(i == i);
            TEST_CHECK(! (i != i));
            TEST_CHECK(i_end == i_end);
            TEST_CHECK(! (i_end != i_end));
        }
    } test_case_libwrapiter_empty_pair;

    /**
     * Implementation details for MyClass.
     */
    struct MyClass::Impl
    {
        std::vector<int> v;
    };

    MyClass::MyClass() :
        _impl(new Impl)
    {
    }

    MyClass::~MyClass()
    {
        delete _impl;
    }

    MyClass::Iterator
    MyClass::begin() const
    {
        return Iterator(_impl->v.begin());
    }

    MyClass::Iterator
    MyClass::end() const
    {
        return Iterator(_impl->v.end());
    }

    /**
     * Implementation details for MyPairClass.
     */
    struct MyPairClass::Impl
    {
        std::map<std::string, std::string> m;
    };

    MyPairClass::MyPairClass() :
        _impl(new Impl)
    {
    }

    MyPairClass::~MyPairClass()
    {
        delete _impl;
    }

    MyPairClass::Iterator
    MyPairClass::begin() const
    {
        return Iterator(_impl->m.begin());
    }

    MyPairClass::Iterator
    MyPairClass::end() const
    {
        return Iterator(_impl->m.end());
    }
}


